package com.supervision.police.controller;

import com.supervision.common.domain.R;
import com.supervision.police.domain.NotePrompt;
import com.supervision.police.domain.NoteRecords;
import com.supervision.police.dto.ListDTO;
import com.supervision.police.service.ModelRecordTypeService;
import com.supervision.police.service.RecordService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;
import java.util.List;

@RestController
@Slf4j
@RequestMapping("record")
public class RecordController {

    @Autowired
    public ModelRecordTypeService modelRecordTypeService;

    @Autowired
    public RecordService recordService;

    /**
     * 查询笔录类型
     * @param name
     * @param page
     * @param size
     * @return
     */
    @GetMapping("queryType")
    public R<?> queryType(@RequestParam(required = false) String name,
                          @RequestParam(required = false, defaultValue = "1") Integer page,
                          @RequestParam(required = false, defaultValue = "20") Integer size) {
        return modelRecordTypeService.queryType(name, page, size);
    }

//    @PostMapping("saveType")
//    public R<?> saveType(@RequestBody ModelRecordType type) {
//        return modelRecordTypeService.saveType(type);
//    }

    /**
     * 保存提示词
     * @param prompt
     * @return
     */
    @PostMapping("/addOrUpdPrompt")
    public R<?> addOrUpdPrompt(@RequestBody NotePrompt prompt) {
        return modelRecordTypeService.addOrUpdPrompt(prompt);
    }

    /**
     * 删除提示词
     * @param prompt
     * @return
     */
    @PostMapping("/delPrompt")
    public R<?> delPrompt(@RequestBody NotePrompt prompt) {
        return modelRecordTypeService.delPrompt(prompt);
    }

    /**
     * 获取案件三元组信息
     * @param caseId
     * @param name
     * @param recordId
     * @return
     */
    @GetMapping("/getThreeInfo")
    public R<?> getThreeInfo(@RequestParam String caseId,
                             @RequestParam String name,
                             @RequestParam(required = false) String recordId) {
        return modelRecordTypeService.getThreeInfo(caseId, name, recordId);
    }

    @PostMapping("/addNeo4j")
    public R<?> addNeo4j(@RequestBody ListDTO list) {
        return modelRecordTypeService.addNeo4j(list.getIds());
    }

    /**
     * 上传笔录， 修改
     * @param records
     * @return
     */
    @PostMapping("/addOrUpdRecords")
    public R<?> uploadRecords(NoteRecords records,
                              @RequestPart("file") List<MultipartFile> fileList) throws IOException {
        return recordService.uploadRecords(records, fileList);
    }

    /**
     * 查询笔录，按姓名为父目录
     * @param noteRecords
     * @param page
     * @param size
     * @return
     */
    @PostMapping("/queryRecords")
    public R<?> queryRecords(@RequestBody NoteRecords noteRecords,
                             @RequestParam(required = false, defaultValue = "1") Integer page,
                             @RequestParam(required = false, defaultValue = "20") Integer size) {
        return recordService.queryRecords(noteRecords, page, size);
    }

    /**
     * 删除
     * @param id
     * @return
     */
    @PostMapping("/delRecords")
    public R<?> delRecords(@RequestParam String id) {
        return recordService.delRecords(id);
    }


}
