package com.supervision.police.service.impl;

import com.alibaba.druid.util.StringUtils;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.supervision.common.domain.R;
import com.supervision.neo4j.domain.CaseNode;
import com.supervision.neo4j.domain.Rel;
import com.supervision.neo4j.service.Neo4jService;
import com.supervision.police.domain.ModelRecordType;
import com.supervision.police.domain.NoteRecord;
import com.supervision.police.domain.NotePrompt;
import com.supervision.police.domain.TripleInfo;
import com.supervision.police.mapper.ModelRecordTypeMapper;
import com.supervision.police.mapper.NoteRecordMapper;
import com.supervision.police.mapper.NotePromptMapper;
import com.supervision.police.mapper.TripleInfoMapper;
import com.supervision.police.service.ModelRecordTypeService;
import lombok.extern.slf4j.Slf4j;
import org.json.JSONArray;
import org.json.JSONObject;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.Generation;
import org.springframework.ai.chat.messages.Message;
import org.springframework.ai.chat.messages.SystemMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StopWatch;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class ModelRecordTypeServiceImpl extends ServiceImpl<ModelRecordTypeMapper, ModelRecordType> implements ModelRecordTypeService {

    @Autowired
    private ModelRecordTypeMapper modelRecordTypeMapper;

    @Autowired
    private NoteRecordMapper noteRecordMapper;

    @Autowired
    private NotePromptMapper notePromptMapper;

    private final OllamaChatClient chatClient;
    @Autowired
    public ModelRecordTypeServiceImpl(OllamaChatClient chatClient) {
        this.chatClient = chatClient;
    }

    @Autowired
    private TripleInfoMapper tripleInfoMapper;

    @Autowired
    private Neo4jService neo4jService;

    @Override
    public R<?> queryType(String name, Integer page, Integer size) {
//        IPage<ModelRecordType> iPage = new Page<>(page, size);
//        iPage  = modelRecordTypeMapper.selectByName(iPage, name);
//        return R.ok(IPages.buildDataMap(iPage));
        List<ModelRecordType> list = modelRecordTypeMapper.selectByName(name);

        for (ModelRecordType modelRecordType : list) {
            //笔录内容
            List<NoteRecord> noteRecords = noteRecordMapper.selectByRecordType(modelRecordType.getRecordType());
            modelRecordType.setRecords(noteRecords);
            //提示词
            List<NotePrompt> prompts = notePromptMapper.queryPrompt(modelRecordType.getId());
            modelRecordType.setPrompts(prompts);
        }
        return R.ok(list);
//        return R.ok(IPages.buildDataMap(iPage));
    }

    @Override
    public ModelRecordType queryByName(String content) {
        Wrapper<ModelRecordType> wrapper = new QueryWrapper<ModelRecordType>().eq("record_type", content);
        return modelRecordTypeMapper.selectOne(wrapper);
    }

    @Override
    public R<?> saveType(ModelRecordType type) {
        String id = type.getId();
        int i = 0;
        if (StringUtils.isEmpty(id)) {
            //新增
            i = modelRecordTypeMapper.insert(type);
        } else {
            i = modelRecordTypeMapper.updateById(type);
        }
        if (i > 0) {
            return R.ok("保存成功");
        } else {
            return R.fail("保存失败");
        }
    }

    @Override
    public R<?> addOrUpdPrompt(NotePrompt prompt) {
        int i = 0;
        if (StringUtils.isEmpty(prompt.getId())) {
            i = notePromptMapper.insert(prompt);
        } else {
            i = notePromptMapper.updateById(prompt);
        }
        if (i > 0) {
            return R.ok("保存成功");
        } else {
            return R.fail("保存失败");
        }
    }

    @Override
    public R<?> delPrompt(NotePrompt prompt) {
        String id = prompt.getId();
        int i = notePromptMapper.deleteById(id);
        if (i > 0) {
            return R.ok("删除成功");
        } else {
            return R.fail("删除失败");
        }
    }

    @Override
    public R<?> getThreeInfo(String caseId, String name, String recordId) {
        List<NoteRecord> records = noteRecordMapper.selectRecord(caseId, name, recordId);
        List<TripleInfo> tripleInfos = new ArrayList<>();
        for (NoteRecord record : records) {
            List<NotePrompt> prompts = notePromptMapper.queryPrompt(record.getRecordTypeId());
            for (NotePrompt prompt : prompts) {
                if (StringUtils.isEmpty(prompt.getPrompt())) {
                    continue;
                }
                try {
                    // 分析三元组
                    List<Message> messages = new ArrayList<>(List.of(new SystemMessage(prompt.getPrompt() + record.getQuestion() + record.getAnswer())));
                    Prompt ask = new Prompt(messages);
                    StopWatch stopWatch = new StopWatch();
                    stopWatch.start();
                    log.info("开始分析:");
                    ChatResponse call = chatClient.call(ask);
                    stopWatch.stop();
                    log.info("耗时:{}", stopWatch.getTotalTimeSeconds());
                    Generation result = call.getResult();
                    String content = result.getOutput().getContent();
                    log.info("分析的结果是:{}", content);
                    JSONObject jsonObject = new JSONObject(content);
                    JSONArray threeInfo = jsonObject.getJSONArray("result");
                    for (int i = 0; i < threeInfo.length(); i++) {
                        JSONObject object = threeInfo.getJSONObject(i);
                        String startNodeType = object.getString("startNodeType");
                        String entity = object.getString("entity");
                        String endNodeType = object.getString("endNodeType");
                        String property = object.getString("property");
                        String value = object.getString("value");
                        //去空
                        if (StringUtils.isEmpty(startNodeType) || StringUtils.isEmpty(entity) || StringUtils.isEmpty(endNodeType)
                                || StringUtils.isEmpty(property) || StringUtils.isEmpty(value)) {
                            continue;
                        }
                        TripleInfo tripleInfo = new TripleInfo(entity, property, value, record.getId(), new Date(), startNodeType, endNodeType);
                        tripleInfoMapper.insert(tripleInfo);
                        tripleInfos.add(tripleInfo);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        }
        return R.ok(tripleInfos);
    }

    @Override
    public R<?> addNeo4j(List<String> ids) {
        List<TripleInfo> tripleInfos = tripleInfoMapper.selectByIds(ids);
        int i = 0;
        for (TripleInfo tripleInfo : tripleInfos) {
            try {
                //开始节点
                String start = tripleInfo.getStartNode();
                CaseNode startNode = neo4jService.findOneByName(tripleInfo.getCaseId(), tripleInfo.getNoteRecordsId(), tripleInfo.getStartNodeType(), start, "1");
                if (startNode == null) {
                    startNode = new CaseNode(start, tripleInfo.getStartNodeType(), tripleInfo.getNoteRecordId(), tripleInfo.getNoteRecordsId(), tripleInfo.getCaseId(), "1");
                    CaseNode save = (CaseNode) neo4jService.save(startNode).getData();
                    startNode.setId(save.getId());
                }
                //结束节点
                String end = tripleInfo.getEndNode();
                CaseNode endNode = neo4jService.findOneByName(tripleInfo.getCaseId(), tripleInfo.getNoteRecordsId(), tripleInfo.getEndNodeType(), end, "1");
                if (endNode == null) {
                    endNode = new CaseNode(end, tripleInfo.getEndNodeType(), tripleInfo.getNoteRecordId(), tripleInfo.getNoteRecordsId(), tripleInfo.getCaseId(), "1");
                    CaseNode save = (CaseNode) neo4jService.save(endNode).getData();
                    endNode.setId(save.getId());
                }
                //关系
                Rel rel = new Rel(startNode.getId(), tripleInfo.getRelation(), endNode.getId());
                Rel relation = neo4jService.findRelation(rel);
                if (relation == null) {
                    R<?> r = neo4jService.saveRelation(rel);
                }
                tripleInfo.setAddNeo4j("1");
                int j = tripleInfoMapper.updateById(tripleInfo);
                if (j > 0) {
                    i++;
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return R.ok("成功插入" + i + "条信息");
    }
}
