package com.supervision.springaidemo.controller;

import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.StrUtil;
import com.supervision.springaidemo.domain.ModelMetric;
import com.supervision.springaidemo.dto.QARecordNodeDTO;
import com.supervision.springaidemo.service.ModelMetricService;
import com.supervision.police.service.NoteCheckRecordService;
import com.supervision.springaidemo.thread.RunCheckThread;
import com.supervision.springaidemo.thread.RunCheckThreadPool;
import com.supervision.springaidemo.util.RecordRegexUtil;
import com.supervision.springaidemo.util.WordReadUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.ai.chat.messages.AssistantMessage;
import org.springframework.ai.chat.messages.Message;
import org.springframework.ai.chat.messages.SystemMessage;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 自己测试的方案,计划改成2阶段的形式
 * 第一阶段,首先根据提示词,从原文中获取笔录的内容(首先让大模型从原文中提取出来可能涉及的内容)
 * 然后再尝试跑结果
 */
@RestController
@Slf4j
@RequestMapping("newTestChat")
public class NewTestController {

    private final OllamaChatClient chatClient;

    @Autowired
    private ModelMetricService modelMetricService;

    @Autowired
    private NoteCheckRecordService noteCheckRecordService;

    @Autowired
    public NewTestController(OllamaChatClient chatClient) {
        this.chatClient = chatClient;
    }

    private static final String template = """
            请根据以下关键词，从笔录中找到相关的原文段落：
            ---
            关键词1:财物去向
            ---
            笔录内容如下：
            ---
            {context}
            ---
            请判断笔录和关键词是否可能有关联,如有关联,则对这段话进行总结
            必须以json数组格式回复
            ---
            {"match":"true/false(相关回复true,不相关回复false)","summary":"如果有关联,则对该笔录进行总结"}
            ---
            好了,现在可以回复了。不要胡编乱造!无中生有!
            """;


    @GetMapping("newTestChat")
    public void newTestChat() {
        // 只查入罪指标
        ModelMetric modelMetric = modelMetricService.lambdaQuery().eq(ModelMetric::getMetricCode, "RZ007").one();
        File file = FileUtil.file("/Users/flevance/Desktop/宁夏审讯大模型/裴金禄/行为人和受害人/裴金禄第一次.docx");
        String context = WordReadUtil.readWord(file.getPath());
        List<QARecordNodeDTO> qaList = RecordRegexUtil.recordRegex(context, "裴金禄");
        for (QARecordNodeDTO qaRecordNodeDTO : qaList) {
            String systemPrompt = """
                     你是一个善于内容提取的模型,能够根据给定关键字从原文中找到可能匹配的原文段落。
                    """;
            List<Message> messages = new ArrayList<>(List.of(new SystemMessage(systemPrompt)));
            Map<String, Object> param = new HashMap<>();
            param.put("context", qaRecordNodeDTO.toString());
            String format = StrUtil.format(template, param);
            List<Message> userMessageList = new ArrayList<>();
            if (format.length() > 8000) {
                log.info("分段提交");
                for (String s : StrUtil.split(format, 6000)) {
                    userMessageList.add(new UserMessage(s));
                    userMessageList.add(new AssistantMessage("继续"));
                }
                userMessageList.remove(userMessageList.size() - 1);
            } else {
                userMessageList.add(new UserMessage(format));
            }
            messages.addAll(userMessageList);

            RunCheckThread runCheck = new RunCheckThread("尝试分段提取-第一段找相似", chatClient, noteCheckRecordService, new Prompt(messages), FileUtil.getName(file), format, systemPrompt, modelMetric, 5);
            RunCheckThreadPool.chatExecutor.submit(runCheck);
        }
    }


}
