package com.supervision.springaidemo.controller;

import cn.hutool.core.map.MapUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONUtil;
import com.supervision.springaidemo.dto.AnswerDTO;
import com.supervision.springaidemo.dto.GlobalResult;
import com.supervision.springaidemo.util.TtsUtil;
import org.springframework.ai.chat.ChatResponse;
import org.springframework.ai.chat.Generation;
import org.springframework.ai.chat.messages.AssistantMessage;
import org.springframework.ai.chat.messages.Message;
import org.springframework.ai.chat.messages.SystemMessage;
import org.springframework.ai.chat.messages.UserMessage;
import org.springframework.ai.chat.prompt.Prompt;
import org.springframework.ai.ollama.OllamaChatClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

/**
 * 测试如何避免出现答非所问
 */
@RequestMapping("vp-test")
@RestController
public class VPTestController {

    private final OllamaChatClient chatClient;


    // 使用多线程进行提交


    @Autowired
    public VPTestController(OllamaChatClient chatClient) {
        this.chatClient = chatClient;
    }

    public static final String userPromptTemplate = """
            现在希望你从预设的问题中,找到与给定问题的意图最匹配的项,以避免出现答非所问
            ---
            给定问题:{userQuestion}
            ---
            预设问题1:{dataBaseQuestion1},问题ID:1761963909201268731
            预设问题2:{dataBaseQuestion2},问题ID:1761963909201268732
            预设问题3:{dataBaseQuestion3},问题ID:1761963909201268733
            预设问题4:{dataBaseQuestion4},问题ID:1761963909201268734
            预设问题5:{dataBaseQuestion5},问题ID:1761963909201268735
            ---
            回复格式为json:{"id":"对应的问题ID","question":"匹配到的预设问题"}
            """;



//    private static final String systemPrompt = """
//            你现在扮演一个医生,以医生的角度从预设问题列表中找到与给定问题最匹配的一项
//            """;

    @GetMapping("test")
    public String test(String userQuestion) {
        List<Message> messageHistoryList = new ArrayList<>();
//        messageHistoryList.add(new SystemMessage(systemPrompt));
        HashMap<String, String> map = new HashMap<>();
        map.put("userQuestion", userQuestion);
        map.put("dataBaseQuestion1", "您有什么症状？");
        map.put("dataBaseQuestion2", "您有什么不舒服的？");
        map.put("dataBaseQuestion3", "您是否有腹痛？");
        map.put("dataBaseQuestion4", "您还有什么不舒服的？");
        map.put("dataBaseQuestion5", "您还有其他什么症状吗？");
        StrUtil.format(userPromptTemplate, map);
        messageHistoryList.add(new UserMessage(StrUtil.format(userPromptTemplate, map)));
        Prompt prompt = new Prompt(messageHistoryList);
        ChatResponse call = chatClient.call(prompt);
        Generation result = call.getResult();
        return result.getOutput().getContent();
    }

}
